CREATE_AGENT_V2 = "api/agents"  # POST -> Create agent
LIST_AGENTS_V2 = "api/agents?status={status}&start={start}&count={count}&accessScope={accessScope}&allowDrafts={allowDrafts}&allowExternal={allowExternal}"  # GET -> List agents
